package com.googlecode.etlunit.module_mojo;

import org.apache.maven.plugin.AbstractMojo;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.project.MavenProject;
import org.apache.maven.project.MavenProjectHelper;
import org.eclipse.jgit.api.Git;
import org.eclipse.jgit.api.errors.GitAPIException;
import org.eclipse.jgit.lib.Repository;
import org.eclipse.jgit.lib.RepositoryBuilder;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Collections;
import java.util.List;
import java.util.Properties;

/**
 * Goal which touches a timestamp file.
 *
 * @goal package-meta-info
 * @phase process-sources
 */
public class EtlUnitModuleMOJO
		extends AbstractMojo
{
	/**
	 * Location of the file.
	 *
	 * @parameter expression="${outputDirectory}" default-value="${project.build.directory}/generated-sources/bitbucket/"
	 * @required
	 */
	private File srcDirectory;

	/**
	 * @parameter expression="${project}" default-value="${project}"
	 */
	protected MavenProject mavenProject;

	/**
	 * Helper class to assist in attaching artifacts to the project instance.
	 * project-helper instance, used to make addition of resources simpler.
	 * @component
	 * @required
	 * @readonly
	 */
	private MavenProjectHelper projectHelper;

	public void execute()
			throws MojoExecutionException
	{
		File f = srcDirectory;

		if (!f.exists())
		{
			f.mkdirs();
		}

		Properties props = new Properties();

		String directory = "META-INF/etlunit-module";
		List includes = Collections.singletonList("**/*");
		List excludes = null;

		projectHelper.addResource(mavenProject, directory, includes, excludes);

		String moduleId = mavenProject.getGroupId() + "|" + mavenProject.getArtifactId();
		String moduleProperties = mavenProject.getGroupId() + "." + mavenProject.getArtifactId() + ".properties";

		props.put("etlunit-module", moduleId);
		props.put("etlunit-module-propertes", moduleProperties);

		try
		{
			FileOutputStream fout = new FileOutputStream(new File(srcDirectory, "etlunit-module.properties"));

			try
			{
				props.store(fout, "");
			}
			finally
			{
				fout.close();
			}

			props.clear();

			Repository repo = new RepositoryBuilder().findGitDir(mavenProject.getBasedir()).build();
			String branch = repo.getFullBranch();

			Git git = new Git(repo);

			props.put("maven.project.groupId", mavenProject.getGroupId());
			props.put("maven.project.artifactId", mavenProject.getArtifactId());
			props.put("maven.project.version", mavenProject.getVersion());

			props.put("git.branch", branch);
			props.put("git.branch.clean", String.valueOf(git.status().call().isClean()));
			props.put("git.branch.version", repo.resolve(repo.getFullBranch()).getName());

			fout = new FileOutputStream(new File(srcDirectory, moduleProperties));

			try
			{
				props.store(fout, "");
			}
			finally
			{
				fout.close();
			}
		}
		catch(IOException exc)
		{
			throw new MojoExecutionException("Error saving properties file", exc);
		}
		catch (GitAPIException e)
		{
			throw new MojoExecutionException("Error reading GIT properties", e);
		}
	}
}